/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.dom;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;

import org.dm.webkit.DMWebKitView;

public class NodeWrapper extends AccessorHolder implements Node{
    DMWebKitView owner;
    String childNodesAccessor;
    String nodeName;
    NamedNodeMapWrapper attributes;
    
    NodeListWrapper children;

    Node parentNode;
    Node previousSibling;
    Node nextSibling;
    
    DOMDocument ownerDocument;
    
    //public NodeWrapper(DMWebKitView owner,String nodeAccessor,int nodeType){
    //}
    
    public NodeWrapper(DMWebKitView owner,String nodeAccessor,short nodeType,DOMDocument ownerDocument,Node parentNode){
        this.parentNode = parentNode;
        this.nodeAccessor = nodeAccessor;
        this.ownerDocument = ownerDocument;
        this.owner = owner;
        this.nodeType = nodeType;
        redefineChildNodes();        
        createAttributes();
    }
    

    public String getCreateString(){return null;}

    protected boolean isOrphan(){
        return (nodeAccessor == null);
    }
    
    public int getChildIndex(Node node){
        if(node == null) return -1;
        if(children == null) return -1;
        for(int i = 0; i < children.getLength(); i++){
            if(node == children.item(i)) return i;
        }
        return -1;
    }
    
    protected void redefineChildNodes(){
        if(nodeType == TEXT_NODE) normalize();
        childNodesAccessor = JSAccessors.getObjectPropertyAccessor(nodeAccessor,"childNodes");
        int n = JSAccessors.getArrayLength(owner,childNodesAccessor);
        nodeName = JSAccessors.getProperty(owner,nodeAccessor,"nodeName");
        nodeType = (short)JSAccessors.getIntFromString(JSAccessors.getProperty(owner,nodeAccessor,"nodeType"));
        NodeWrapper []children0 = null;
        if(n > 0){
            children0 = new NodeWrapper[n];
        }
        for(int i = 0; i < n; i++){
            String accessor = JSAccessors.getObjectArrayAccessor(nodeAccessor,"childNodes",i);
            short type = JSAccessors.getNodeType(owner,accessor);
            children0[i] = NodeFactory.createWrapper(owner,accessor,type,ownerDocument,this);
            if(i > 0){
                children0[i - 1].nextSibling = children0[i];
                children0[i].previousSibling = children0[i-1];
            }
        }
        children = new NodeListWrapper(children0);
    }
    
    protected void createAttributes(){
        attributes = new NamedNodeMapWrapper(owner,JSAccessors.getObjectPropertyAccessor(nodeAccessor,"attributes"));
    }
    
    public void normalize(){
        owner.runJS(JSAccessors.getObjectMethodAccessor(nodeAccessor,"normalize"));
    }
    
    public int getChildrenCount(){
        return (children == null)?0:children.getLength();
    }
    
    public String getNodeName(){
        return nodeName;
    }

    public String getLocalName(){
        return getNodeName();
    }

    
    public short getNodeType(){
        return nodeType;
    }
    
    public NodeWrapper getNodeWrapper(int i){
        if(i < 0 || i >= getChildrenCount()) return null;
        return (NodeWrapper)children.item(i);
    }
    public void printNode(){
        printNode("");
    }
    public void printNode(String prefix){
        System.out.println(prefix+toString());
        prefix += "    ";
        for(int i = 0; i < getChildrenCount(); i++){
           NodeWrapper nw =  getNodeWrapper(i);
           if(nw == null) continue;
           nw.printNode(prefix);
        }
    }

    public void testNode(Document document){
        for(int i = 0; i < getChildrenCount(); i++){
           NodeWrapper nw =  getNodeWrapper(i);
           if(nw == null) continue;
           if(nw instanceof TextWrapper){
                TextWrapper text = (TextWrapper)nw;
                //((TextWrapper)nw).appendData("12345678");
                if(text.getData().startsWith("zzzzz")){//
                    //org.w3c.dom.Text newText = text.splitText(6);
                    //System.out.println("newText "+newText.getData());
                    if(org.dm.webkit.WebKitJava.testCount < 1){
                        org.w3c.dom.Element newElement = document.createElement("FONT");
                        newElement.setAttribute("color","808080");
                        newElement.setAttribute("size","+2");
                        //Node  retElement = insertBefore(newElement,text);
                        int index = getChildIndex(text);
                        Node replacedNode = replaceChild(newElement,text);
                        Node updatedNode = getChildNodes().item(index);
                        updatedNode.appendChild(replacedNode);
                        //org.w3c.dom.Text newText = document.createTextNode("dima test");
                        //Node  retText = retElement.appendChild(newText);
                        //System.out.println("retText "+retText);
                        org.dm.webkit.WebKitJava.testCount = 1;
                    }
                }
           }
           nw.testNode(document);
        }
    }

    public String toString(){
        return getNodeName();
    }

    public NamedNodeMap getAttributes(){
        return attributes;
    }
    
    public boolean hasAttributes(){
        return (attributes != null);
    }
    
    public  static String getNodeTypeAsString(int nodet){
        switch(nodet){
            case ELEMENT_NODE:                  return "Element";
            case ATTRIBUTE_NODE:                return "Attribute";
            case TEXT_NODE:                     return "Text";
            case CDATA_SECTION_NODE:            return "CDATA";
            case ENTITY_REFERENCE_NODE:         return "Entity Reference";
            case ENTITY_NODE:                   return "Entity";
            case PROCESSING_INSTRUCTION_NODE:   return "Processing Instruction";
            case COMMENT_NODE:                  return "Comment";
            case DOCUMENT_NODE:                 return "Document";
            case DOCUMENT_TYPE_NODE:            return "Document Type";
            case DOCUMENT_FRAGMENT_NODE:        return "Document Fragment";
            case NOTATION_NODE:                 return "Notation";
        }
        return "Undefined";
    }

    public void setNodeValue(String nodeValue) throws DOMException{
        owner.runJS(getNodeAccessor()+".nodeValue=\""+nodeValue+"\"");
    }

    public String getNodeValue() throws DOMException{
        return JSAccessors.getProperty(owner,getNodeAccessor(),"nodeValue");
    }


    public Node getParentNode(){
        return parentNode;
    }

    public NodeList getChildNodes(){
        return children;
    }

    public Node getFirstChild(){
        if(!hasChildNodes()) return null;
        return children.item(0);
    }
    
    public Node getLastChild(){
        if(!hasChildNodes()) return null;
        return children.item(children.getLength() - 1);
    }
    
    public Document getOwnerDocument(){
        return ownerDocument;
    }

    public Node getPreviousSibling(){
        return previousSibling;
    }

    public Node getNextSibling(){
        return nextSibling;
    }

    public Node insertBefore(Node newChild, Node refChild) throws DOMException{
        if(newChild == null) throw new DOMException(DOMException.NOT_FOUND_ERR,"newChild in insertBefore couldn't be null");
        if(isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"inserting node into the \"orphan\" node isn't supported");
        
        int myIndex = (refChild == null)?getChildrenCount():-1;
        if(parentNode instanceof NodeWrapper){
            if(refChild != null) myIndex = getChildIndex(refChild);
        }
        String refAccessor = (refChild == null)?null:((NodeWrapper)refChild).getNodeAccessor();
        if(!((NodeWrapper)newChild).isOrphan()){
            String newChildAccessor = ((NodeWrapper)newChild).getNodeAccessor();
            JSAccessors.callMethod(owner,nodeAccessor,"insertBefore",new String[]{newChildAccessor,refAccessor});
        }else{
            String createString = ((NodeWrapper)newChild).getCreateString();
            if(createString == null) return null;
            StringBuffer sb = new StringBuffer(createString);
            sb.append(nodeAccessor+".insertBefore(newNode,"+refAccessor+")\n");
            owner.runJS(sb.toString());
        }
        redefineChildNodes();
        if(myIndex != -1){
            NodeList children = getChildNodes();
            return children.item(myIndex);
        }
        return null;
    }

    public Node appendChild(Node newChild) throws DOMException{
        if(newChild == null) throw new DOMException(DOMException.NOT_FOUND_ERR,"newChild in appendChild couldn't be null");
        if(isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"appending new child into the \"orphan\" node isn't supported");
        return insertBefore(newChild,null);
    }

    public Node replaceChild(Node newChild, Node oldChild) throws DOMException{
        if(newChild == null) throw new DOMException(DOMException.NOT_FOUND_ERR,"newChild in replaceChild couldn't be null");
        if(oldChild == null) throw new DOMException(DOMException.NOT_FOUND_ERR,"oldChild in replaceChild couldn't be null");
        if(isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"replacing  child in the \"orphan\" node isn't supported");
        if(((NodeWrapper)oldChild).isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"oldChild in replaceChild couldn't be  \"orphan\" node");
        Node newNode = insertBefore(newChild,oldChild);
        Node removedNode = removeChild(newNode.getNextSibling());
        return removedNode;
    }

    public Node removeChild(Node oldChild) throws DOMException{
        if(oldChild == null) throw new DOMException(DOMException.NOT_FOUND_ERR,"oldChild in removeChild couldn't be null");
        if(isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"removing  child from the \"orphan\" node isn't supported");
        if(((NodeWrapper)oldChild).isOrphan()) throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,"oldChild in removeChild couldn't be  \"orphan\" node");
        Node removedNode = oldChild;
        StringBuffer sb = new StringBuffer();
        sb.append(nodeAccessor+".removeChild("+((NodeWrapper)oldChild).getNodeAccessor()+")\n");
        owner.runJS(sb.toString());
        redefineChildNodes();
        ((NodeWrapper)removedNode).setNodeAccessor(null);
        ((NodeWrapper)removedNode).parentNode = null;
        return removedNode;
    }
    public boolean hasChildNodes(){
        return (children != null && children.getLength() > 0);
    }

    public Node cloneNode(boolean deep){
        throw new RuntimeException("cloneNode isn't implemented");
    }

    public boolean isSupported(String feature, String version){
        throw new RuntimeException("isSupported isn't implemented");
    }

    public String getNamespaceURI(){
        throw new RuntimeException("getNamespaceURI isn't implemented");
    }

    public String getPrefix(){
        throw new RuntimeException("getPrefix isn't implemented");
    }

    public void setPrefix(String prefix) throws DOMException{
        throw new RuntimeException("setPrefix isn't implemented");
    }

    public static final int UNDEFINED_NODE = 0;
    short nodeType = UNDEFINED_NODE;
}

